package com.nisum.ecomcustomer.controller;

import com.nisum.ecomcustomer.model.Customer;
import com.nisum.ecomcustomer.service.CustomerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotNull;
import java.util.List;

@RestController
@RequestMapping("/customer")
@Validated
public class CustomerController {

    @Autowired
    CustomerService customerService;

    @PostMapping("/register")
    public ResponseEntity<Customer> registerCustomer(@RequestBody @Valid @NotNull(message = "body can't be empty") Customer customer) {
        return new ResponseEntity<>(customerService.register(customer), HttpStatus.CREATED);
    }

    @PostMapping("/registerAll")
    public ResponseEntity<List<Customer>> registerCustomers(@RequestBody @NotNull @Valid List<Customer> customer) {
        return new ResponseEntity<>(customerService.registerAll(customer), HttpStatus.CREATED);
    }

    @PutMapping("/update")
    public ResponseEntity<Customer> updateCustomer(@RequestBody @NotNull(message = "body can't be empty") @Valid Customer customer) {
        return new ResponseEntity<>(customerService.update(customer), HttpStatus.CREATED);
    }

    @GetMapping("/")
    public ResponseEntity<List<Customer>> getAllCustomers() {
        return ResponseEntity.ok(customerService.getAllCustomers());
    }

    @GetMapping("/id/{id}")
    public ResponseEntity<Customer> getCustomerById(@PathVariable Long id) {
        return ResponseEntity.ok(customerService.getCustomerById(id));
    }

    //TODO - make email field unique

    //    @GetMapping("/email/{email}")
    public ResponseEntity<Customer> getCustomerByEmail(@PathVariable @Email String email) {
        return ResponseEntity.ok(customerService.getCustomerByEmail(email));
    }

    @GetMapping("/fname/{fname}")
    public ResponseEntity<List<Customer>> getCustomersByFirstName(@PathVariable("fname") String fName) {
        return ResponseEntity.ok(customerService.getCustomerByFirstName(fName));
    }

    @GetMapping("/lname/{lname}")
    public ResponseEntity<List<Customer>> getCustomersByLastName(@PathVariable("lname") String lName) {
        return ResponseEntity.ok(customerService.getCustomerByLastName(lName));
    }

    @DeleteMapping("/id/{id}")
    public ResponseEntity<Void> deleteById(@PathVariable Long id) {
        customerService.deleteById(id);
        return ResponseEntity.noContent().build();
    }
}
